#ifndef AVCODEC_MPEG4Z_H
#define AVCODEC_MPEG4Z_H

#ifdef ZAV
#include "brcz.h"
#endif

///////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////

#define MEP_MAX_PICTURE_COUNT       3

//////////////////////////////////////////////////////////////////////////////
// Start code markers
//////////////////////////////////////////////////////////////////////////////

#define VOL_STARTCODE               0x120
#define VOS_STARTCODE               0x1B0
#define USER_DATA_STARTCODE         0x1B2
#define GOP_STARTCODE               0x1B3
#define VISUAL_OBJ_STARTCODE        0x1B5
#define VOP_STARTCODE               0x1B6

//////////////////////////////////////////////////////////////////////////////
// 
//////////////////////////////////////////////////////////////////////////////

#define RECT_SHAPE                  0

#define SIMPLE_VO_TYPE              1
#define ADV_SIMPLE_VO_TYPE          17

#define PIXEL_ASPECT_FIRST          1
#define PIXEL_ASPECT_LAST           6


///////////////////////////////////////////////////////////////////////////////
// The following structure contains an information for frame encoding/decoding
// by mpeg4z codec.
// This structure is used as AVCODECCONTEXT private data for mpeg4z codec.
///////////////////////////////////////////////////////////////////////////////

typedef struct Mpeg4zEncContext 
{
    struct AVCodecContext *avctx;
    
    int codec_id;

    // If 1 - encoding mode
    // If 0 - decoding mode    
    int encoding; 
    
    //
    int flags; 

    // Max number of b-frames for encoding
    int max_b_frames;
                            
    // Encoder bitstream context
    PutBitContext pb;

    // Decoder bitstream context
    GetBitContext gb;

    ParseContext parse_context;

    // Picture buffer
    AVFrame picture[MEP_MAX_PICTURE_COUNT];
                     
    // Pointer to forward reference frame
    AVFrame *last_picture_ptr;

    // Pointer to backward reference frame
    AVFrame *next_picture_ptr;

    // Pointer to currently encoded/decoded frame. 
    AVFrame *current_picture_ptr;

    // Copy of picture, which is currently encoded/decoded.
    // This structure is an application interface, to reveal information,
    // about currently coded picture. (Like picture type)
    AVFrame coded_picture;

    // Picture size, must be a multiple of 16
    int width;
    int height;

    // Number of macro blocks in frame.
    int mb_in_frame; 
 
    // Currently coded picture type: I_TYPE, P_TYPE, B_TYPE
    int pict_type; 

    // MV resolution.
    int f_code;

    // Maximum motion estimation search range in pixel units. The search size
    // is for single side of macro block.
    int me_search_x;            
    int me_search_y;     

#ifdef ZAV
    // Bit Rate Control data
    SBRCCONTEXT brc_ctx;
#endif

    // Indicates for decoder to skip frames.
    int hurry_up;
               
    //
    // :ZZZ: To check if all those fields are actually needed.
    // Mpeg4 standard header markers. 
    //    

    int time_increment_resolution;

    // Number of bits to represent the fractional part of time
    int time_increment_bits; 
    int last_time_base;

    // Time in seconds of last I,P,S Frame
    int time_base;
    
    // Time of current frame
    int64_t time;
    int64_t last_non_b_time;

    // Time distance between the last 2 p,s,i frames 
    uint16_t pp_time;

    // Time distance between the last b and p,s,i frame 
    uint16_t pb_time; 
    
    // Video object layer shape
    int shape;

    // If 1 - quarter sample
    // If 0 - half sample
    int quarter_sample;

    // Pixel aspect ration
    int aspect_ratio_info; //FIXME remove
    int aspected_width;    //FIXME remove
    int aspected_height;   //FIXME remove

    // no reordering needed / has no b-frames
    int low_delay; 
    int vo_type;

    // does the stream contain the low_delay flag, 
    // used to workaround buggy encoders 
    int vol_control_parameters;
 
    // Check if we need it.
    int mpeg_quant;

    // Use mpeg4/h263 ac/dc predictions    
    int h263_pred;
 
    // Set if the next p frame is damaged, to avoid showing trashed b frames
    int next_p_frame_damaged; 

    // divx specific, used to workaround (many) bugs in divx5
    int divx_version;
    int divx_packed;
#define BITSTREAM_BUFFER_SIZE 1024*256
    uint8_t *bitstream_buffer; //Divx 5.01 puts several frames in a single one, this is used to reorder them
    int bitstream_buffer_size;

    // time 
    int dummy_pframe_time;
                                
} Mpeg4zEncContext;


#endif // AVCODEC_MPEG4Z_H
