/**
 * @file mpegaudio.h
 * mpeg audio declarations for both encoder and decoder.
 */

/* max frame size, in samples */
#define MPA_FRAME_SIZE 1152 

/* max compressed frame size */
#define MPA_MAX_CODED_FRAME_SIZE 1792

#define MPA_MAX_CHANNELS 2

#define SBLIMIT 32 /* number of subbands */

#define MPA_STEREO  0
#define MPA_JSTEREO 1
#define MPA_DUAL    2
#define MPA_MONO    3

#if ( FW_VERSION == 0x92 )

// SP 0: samples_buf ( 4 Kbyte )
// SP 1+2: costab32, bitinv32, tmp, tmp1 ( < 1 Kbyte )
// SP 3: filter_bank ( 1 Kbyte )

/*
#define COSTAB32_ADDR    0x90009000 // 30 int = 120 byte
#define BITINV32_ADDR    0x90009078 // 32 int = 128 byte
#define TMP_ADDR         0x900090F8 // 64 int = 256 byte
#define TMP1_ADDR        0x900091F8 // 32 int = 128 byte
#define FILTER_BANK_ADDR 0x90009400 // 512 short = 1024 byte
#define SAMPLES_BUF_ADDR 0x90008000 // 4096 bytes
*/

/*
void idct32_part1( int *tab ) __attribute__ ((section (".spc2")));
void idct32_part2( int* tab, const int* xp ) __attribute__ ((section (".spc2")));
void idct32_part3( int *tab, int *out ) __attribute__ ((section (".spc2")));
*/

#elif ( FW_VERSION == 0x82 )

// SP 0: samples_buf ( 4 Kbyte )
// SP 4: code ( < 1 KByte )
// SP 5: filter_bank, costab32, bitinv32, tmp, tmp1 ( < 2 KByte )

#define COSTAB32_ADDR    0x9000AC00 // 30 int = 120 byte
#define BITINV32_ADDR    0x9000AC78 // 32 int = 128 byte
#define TMP_ADDR         0x9000ACF8 // 64 int = 256 byte
#define TMP1_ADDR        0x9000ADF8 // 32 int = 128 byte
#define FILTER_BANK_ADDR 0x9000A800 // 512 short = 1024 byte
#define SAMPLES_BUF_ADDR 0x90008000 // 4096 bytes

void idct32_part1( int *tab ) __attribute__ ((section (".spc4")));
void idct32_part2( int* tab, const int* xp ) __attribute__ ((section (".spc4")));
void idct32_part3( int *tab, int *out ) __attribute__ ((section (".spc4")));

#elif ( FW_VERSION == 0x72 )

// SP 0: samples_buf ( 4 Kbyte )
// SP 1: code ( < 1 KByte )
// SP 2: filter_bank ( 1 Kbyte )
// SP 3: costab32, bitinv32, tmp, tmp1 ( < 1 Kbyte )

#define COSTAB32_ADDR    0x90009800 // 30 int = 120 byte
#define BITINV32_ADDR    0x90009878 // 32 int = 128 byte
#define TMP_ADDR         0x900098F8 // 64 int = 256 byte
#define TMP1_ADDR        0x900099F8 // 32 int = 128 byte
#define FILTER_BANK_ADDR 0x90009400 // 512 short = 1024 byte
#define SAMPLES_BUF_ADDR 0x90008000 // 4096 bytes

void idct32_part1( int *tab ) __attribute__ ((section (".spc1")));
void idct32_part2( int* tab, const int* xp ) __attribute__ ((section (".spc1")));
void idct32_part3( int *tab, int *out ) __attribute__ ((section (".spc1")));

#else

#define SAMPLES_BUF_ADDR 0
#define FILTER_BANK_ADDR 0
#define COSTAB32_ADDR    costab32
#define BITINV32_ADDR    bitinv32
#define TMP_ADDR         0
#define TMP1_ADDR        0

void idct32_part1( int *tab );
void idct32_part2( int* tab, const int* xp );
void idct32_part3( int *tab, int *out );

#endif



int l2_select_table(int bitrate, int nb_channels, int freq, int lsf);

extern const uint16_t mpa_bitrate_tab[2][3][15];
extern const uint16_t mpa_freq_tab[3];
extern const unsigned char *alloc_tables[5];
extern const double enwindow[512];
extern const int sblimit_table[5];
extern const int quant_steps[17];
extern const int quant_bits[17];
extern const int32_t mpa_enwindow[257];
