/*
 * Raw Video Codec
 * Copyright (c) 2001 Fabrice Bellard.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/**
 * @file raw.c
 * Bypass Video Codec
 */
 
#include "avcodec.h"

/* Bypass Encoder Implementation */

static int bypass_init_decoder(AVCodecContext *avctx)
{
    return 0;
}

static int bypass_decode(AVCodecContext *avctx,
			             void *data,
			             int *data_size,
			             uint8_t *buf,
			             int buf_size)
{
    AVPicture* picture = (AVPicture*)data;

    avpicture_fill( picture,
                    buf,
                    avctx->pix_fmt,
                    avctx->width,
                    avctx->height );

    *data_size = sizeof(AVPicture);

    return buf_size;
}

static int bypass_close_decoder(AVCodecContext *avctx)
{
    return 0;
}

static int bypass_init_encoder(AVCodecContext *avctx)
{
    return 0;
}

static int bypass_encode(AVCodecContext* avctx,
                         unsigned char* frame,
                         int buf_size,
                         void *data)
{
    *(AVFrame*)frame = *(AVFrame*)data;

    return sizeof(AVFrame);
}

AVCodec bypass_encoder = {
    "bypass",
    CODEC_TYPE_VIDEO,
    CODEC_ID_BYPASS,
    0,
    bypass_init_encoder,
    bypass_encode,
};

AVCodec bypass_decoder = {
    "bypass",
    CODEC_TYPE_VIDEO,
    CODEC_ID_BYPASS,
    0,
    bypass_init_decoder,
    NULL,
    bypass_close_decoder,
    bypass_decode,
};
