/*
 * Raw Video Codec
 * Copyright (c) 2001 Fabrice Bellard.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/**
 * @file rawz.c
 * Bypass Video Codec
 */
 
#include "avcodec.h"

static int rawz_init_decoder(AVCodecContext *avctx)
{
    int i;

    avctx->coded_frame = avcodec_alloc_frame();
    avctx->coded_frame->key_frame = 1;
    avctx->coded_frame->pict_type = FF_I_TYPE;
 
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].width        = avctx->width;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].height       = avctx->height;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].left         = 0;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].top          = 0;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].frame_width  = avctx->width; 
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].frame_height = avctx->height;

    return 0;
}

static int rawz_decode(AVCodecContext *avctx,
			           void *data,
			           int *data_size,
			           uint8_t *buf,
			           int buf_size)
{
    AVPicture* picture = (AVPicture*)data;
    char *decoded_buf;

    *data_size = 0;

    if ( buf == NULL )
        return buf_size;
          
    avctx->pix_fmt = PIX_FMT_YUV422;

    av_mem_lock(buf);
    decoded_buf = buf;

    avpicture_fill(picture, 
                   decoded_buf, 
                   avctx->pix_fmt, 
                   avctx->width, 
                   avctx->height);

    avctx->block_buf[AV_BLOCK_BUFFER_YUV].width        = avctx->width;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].height       = avctx->height;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].left         = 0;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].top          = 0;
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].frame_width  = avctx->width; 
    avctx->block_buf[AV_BLOCK_BUFFER_YUV].frame_height = avctx->height;

    picture->data[AV_BUF_FIFO_WRITE] = NULL;

    *data_size = sizeof(AVPicture);

    return buf_size;
}

static int rawz_close_decoder(AVCodecContext *avctx)
{
    av_freep(&avctx->coded_frame);
    return 0;
}

static int rawz_init_encoder(AVCodecContext *avctx)
{    
    avctx->coded_frame = avcodec_alloc_frame();
    avctx->coded_frame->key_frame = 1;
    avctx->coded_frame->pict_type = FF_I_TYPE;

    return 0;
}

static int rawz_encode(AVCodecContext *avctx,
                       unsigned char *frame,
                       int buf_size,
                       void *data)
{
    memcpy(frame, data, buf_size);

    return buf_size;
}


static int rawz_close_encoder(AVCodecContext *avctx)
{
    av_freep(&avctx->coded_frame);
    return 0;
}


AVCodec rawz_encoder = {
    "null",
    CODEC_TYPE_VIDEO,
    CODEC_ID_RAWZ,
    0,
    rawz_init_encoder,
    rawz_encode,
};

AVCodec rawz_decoder = {
    "null",
    CODEC_TYPE_VIDEO,
    CODEC_ID_RAWZ,
    0,
    rawz_init_decoder,
    NULL,
    rawz_close_decoder,
    rawz_decode,
};
